import React, { useState, useEffect } from 'react';
import { Rocket } from 'lucide-react';
import { LocalizationProvider } from '@mui/x-date-pickers';
import { AdapterDateFns } from '@mui/x-date-pickers/AdapterDateFns';
import IdeaGenerator from './components/IdeaGenerator';
import IdeaList from './components/IdeaList';
import RoadmapView from './components/RoadmapView';
import { generateIdeasWithAI } from './utils/api';
import { SavedPlansProvider, useSavedPlans } from './contexts/SavedPlansContext';
import Navbar from './components/Navbar';
import { StartupPlan } from './types/StartupPlan';
import FollowStepsPage from './pages/FollowStepsPage';
import { BrowserRouter, Routes, Route, useLocation, Navigate, useNavigate } from 'react-router-dom';
import RoadmapPage from './pages/RoadmapPage';
import ProductRoadmapPage from './pages/ProductRoadmapPage';
import TechnologyPage from './pages/TechnologyPage';
import RecruitmentPage from './pages/RecruitmentPage';
import MarketingPage from './pages/MarketingPage';
import StorePage from './pages/StorePage';
import { StorePageProvider } from './contexts/StorePageContext';
import Footer from './components/Footer';
import AppsPage from './pages/dashboard/AppsPage';
import FeaturesPage from './pages/FeaturesPage';
import HelpPage from './pages/HelpPage';
import FAQPage from '../purplestartups/src/pages/FAQPage';
import ContactPage from './pages/ContactPage';
import AboutPage from './pages/AboutPage';
import CareersPage from './pages/CareersPage';
import BlogPage from './pages/BlogPage';
import PrivacyPage from './pages/PrivacyPage';
import TermsPage from './pages/TermsPage';
import CookiesPage from './pages/CookiesPage';
import RefundPolicy from './pages/RefundPolicy';
import PricingPage from './pages/PricingPage';
import { AuthProvider, useAuth } from './contexts/AuthContext';
import LoginPage from './pages/LoginPage';
import ProtectedRoute from './components/ProtectedRoute';
import SignupPage from './pages/SignupPage';
import ProfilePage from './pages/ProfilePage';
import SavedIdeasPage from './pages/SavedIdeasPage';
import SecurityPage from './pages/SecurityPage';
import AccountSettingsPage from './pages/AccountSettingsPage';
import DashboardPage from './pages/DashboardPage';
import NotificationsPage from './pages/NotificationsPage';
import BillingPage from './pages/BillingPage';
import DashboardWelcome from './components/DashboardWelcome';
import ApiSettingsPage from './pages/ApiSettingsPage';
import { SubscriptionProvider, useSubscription } from './contexts/SubscriptionContext';
import AdminRoutes from './routes/AdminRoutes';
import { AdminRoute } from './components/AdminRoute';
import { supabase } from './utils/supabaseClient';
import AdminSetupPage from './pages/AdminSetupPage';
import AdminDebugPage from './pages/AdminDebugPage';
import AdminDashboardPage from './pages/AdminDashboardPage';
import AdminLayout from './components/AdminLayout';
import UserManagementPage from './pages/admin/UserManagementPage';
import ContentManagementPage from './pages/admin/ContentManagementPage';
import SettingsPage from './pages/admin/SettingsPage';
import AdminEntryPage from './pages/AdminEntryPage';
import AnalyticsPage from './pages/admin/AnalyticsPage';
import UpgradeModal from './components/UpgradeModal';
import Admin1Routes from './routes/Admin1Routes';
import BlogManagementPage from './pages/admin/BlogManagementPage';
import BlogPostDetailPage from './pages/BlogPostDetailPage';
import ContactMessagesPage from './pages/admin/ContactMessagesPage';
import TestimonialPage from './pages/TestimonialPage';
import { HelmetProvider, Helmet } from 'react-helmet-async';
import NotFoundPage from './pages/NotFoundPage';
import SitemapPage from './pages/SitemapPage';
import { initGA, trackPageView, trackPerformance } from './utils/analytics';
import CookieConsentBanner from './components/CookieConsentBanner';
import IdeasPage from './pages/IdeasPage';
import ForgotPasswordPage from './pages/ForgotPasswordPage';
import ResetPasswordPage from './pages/ResetPasswordPage';
import FinancialProjectionsPage from './pages/FinancialProjectionsPage';
import AIMarketIntelligencePage from './pages/AIMarketIntelligencePage';
import AIMarketingAssistantPage from './pages/AIMarketingAssistantPage';
import AISalesOptimizerPage from './pages/AISalesOptimizerPage';
import AIProductStrategistPage from './pages/AIProductStrategistPage';
import AIOperationsManagerPage from './pages/AIOperationsManagerPage';
import AILegalAssistantPage from './pages/AILegalAssistantPage';
import MarketingRoutes from './routes/MarketingRoutes';
import DocsPage from './pages/DocsPage';
import CommunityForumPage from './pages/CommunityForumPage';
import KnowledgeBasePage from './pages/KnowledgeBasePage';
import TrendingKeywordsManager from './components/admin/TrendingKeywordsManager';
import BusinessAnalystPage from './pages/BusinessAnalystPage';
import BusinessAnalystDetailPage from './pages/BusinessAnalystDetailPage';
import { Session } from '@supabase/supabase-js';

function ScrollToTop() {
  const { pathname } = useLocation();

  useEffect(() => {
    window.scrollTo(0, 0);
    // Track page view when route changes
    trackPageView(pathname, document.title);
  }, [pathname]);

  return null;
}

// Create a new component for the app content
function AppContent() {
  const [vagueConcept, setVagueConcept] = useState('');
  const [ideas, setIdeas] = useState<string[]>([]);
  const [selectedIdea, setSelectedIdea] = useState<string | null>(null);
  const [isLoading, setIsLoading] = useState(false);
  const [currentPlan, setCurrentPlan] = useState<StartupPlan | null>(null);
  const [showSidebar, setShowSidebar] = useState(false);
  const [session, setSession] = useState<Session | null>(null);
  const [upgradeModalOpen, setUpgradeModalOpen] = useState(false);
  const [isPageReady, setIsPageReady] = useState(false);
  
  // Now this hook is used within a Router context
  const { loading: authLoading } = useAuth();
  const navigate = useNavigate();
  const { savedPlans } = useSavedPlans();
  const { canGeneratePlan, subscription, incrementPlansGenerated } = useSubscription();

  // Initialize Google Analytics
  useEffect(() => {
    initGA();
  }, []);

  // Mark page as ready when critical content is loaded
  useEffect(() => {
    if (!authLoading && !isLoading) {
      setIsPageReady(true);
    }
  }, [authLoading, isLoading]);

  // Track performance metrics
  useEffect(() => {
    if ('performance' in window) {
      // Track Speed Index
      const observer = new PerformanceObserver((list) => {
        for (const entry of list.getEntries()) {
          if (entry.name === 'speed-index') {
            trackPerformance('SpeedIndex', entry.startTime);
          }
        }
      });
      observer.observe({ entryTypes: ['speed-index'] });

      // Track First Contentful Paint
      const observerFCP = new PerformanceObserver((list) => {
        for (const entry of list.getEntries()) {
          if (entry.name === 'first-contentful-paint') {
            trackPerformance('FCP', entry.startTime);
          }
        }
      });
      observerFCP.observe({ entryTypes: ['paint'] });

      // Track Largest Contentful Paint
      const lcpObserver = new PerformanceObserver((list) => {
        for (const entry of list.getEntries()) {
          trackPerformance('LCP', entry.startTime);
        }
      });
      lcpObserver.observe({ entryTypes: ['largest-contentful-paint'] });

      // Track First Input Delay
      const fidObserver = new PerformanceObserver((list) => {
        for (const entry of list.getEntries()) {
          trackPerformance('FID', entry.duration);
        }
      });
      fidObserver.observe({ entryTypes: ['first-input'] });

      return () => {
        observer.disconnect();
        observerFCP.disconnect();
        lcpObserver.disconnect();
        fidObserver.disconnect();
      };
    }
  }, []);

  // Keep all your functions and useEffect hooks here
  const handleGenerateIdeas = async (concept: string) => {
    setIsLoading(true);
    
    // Check subscription limit first
    if (!canGeneratePlan) {
      // Show upgrade modal
      setUpgradeModalOpen(true);
      setIsLoading(false);
      return;
    }
    
    try {
      const generatedIdeas = await generateIdeasWithAI(concept);
      setIdeas(generatedIdeas);
      
      // Save each generated idea to Supabase
      const { data: userData } = await supabase.auth.getUser();
      if (!userData?.user) {
        throw new Error('User must be authenticated to save ideas');
      }

      // Save each idea to the ideas table
      for (const idea of generatedIdeas) {
        const { error } = await supabase
          .from('ideas')
          .insert([
            {
              title: idea,
              description: `A startup idea based on the concept: ${concept}`,
              author_id: userData.user.id,
              status: 'active',
              likes: 0,
              metadata: {
                concept: concept,
                generated_at: new Date().toISOString()
              }
            }
          ]);
        
        if (error) {
          console.error('Error saving idea:', error);
        }
      }
      
      // Increment the counter only after successful generation
      await incrementPlansGenerated();
    } catch (error) {
      console.error('Failed to generate ideas:', error);
    } finally {
      setIsLoading(false);
    }
  };

  const handlePlanSelect = (plan: StartupPlan) => {
    setCurrentPlan(plan);
    
    // Update the URL to include the plan ID
    // This could be the plan id or a timestamp or some unique identifier
    if (plan.id) {
      navigate(`/plan/${plan.id}`, { replace: false });
    } else if (plan.timestamp) {
      navigate(`/plan/${plan.timestamp}`, { replace: false });
    }
    
    setShowSidebar(false);
  };

  useEffect(() => {
    // Get initial session
    const getInitialSession = async () => {
      const { data, error } = await supabase.auth.getSession();
      if (error) {
        console.error('Error getting session:', error);
      } else {
        setSession(data.session);
      }
    };
    
    getInitialSession();

    // Set up auth state listener
    const { data: { subscription } } = supabase.auth.onAuthStateChange(
      (_event, session) => {
        setSession(session);
      }
    );

    // Clean up subscription
    return () => {
      subscription.unsubscribe();
    };
  }, []);

  useEffect(() => {
    // Extract plan ID from URL if present
    const pathname = window.location.pathname;
    const match = pathname.match(/\/plan\/(\w+)/);
    
    if (match && match[1]) {
      const planId = match[1];
      
      // Find plan in saved plans or load it from API/database
      if (savedPlans) {
        const foundPlan = savedPlans.find(p => 
          p.id === planId || p.timestamp?.toString() === planId
        );
        
        if (foundPlan) {
          setCurrentPlan(foundPlan);
        } else {
          // If not found in saved plans, try to fetch it from the server
          // This would depend on your API structure
          // fetchPlanById(planId).then(plan => setCurrentPlan(plan));
        }
      }
    }
  }, []);

  // Don't render routes until auth is loaded
  if (authLoading) {
    return (
      <div className="flex h-screen w-full items-center justify-center">
        <div className="text-center">
          <div className="spinner h-12 w-12 animate-spin rounded-full border-4 border-solid border-indigo-600 border-t-transparent mx-auto"></div>
          <p className="mt-4 text-lg">Loading authentication...</p>
        </div>
      </div>
    );
  }

  const websiteStructuredData = {
    "@context": "https://schema.org",
    "@type": "WebSite",
    "name": "Purple Startups",
    "url": window.location.origin,
    "potentialAction": {
      "@type": "SearchAction",
      "target": {
        "@type": "EntryPoint",
        "urlTemplate": `${window.location.origin}/search?q={search_term_string}`
      },
      "query-input": "required name=search_term_string"
    },
    "sameAs": [
      "https://twitter.com/purplestartups",
      "https://www.linkedin.com/company/purple-startups",
      "https://www.facebook.com/purplestartups"
    ]
  };

  const organizationStructuredData = {
    "@context": "https://schema.org",
    "@type": "Organization",
    "name": "Purple Startups",
    "url": window.location.origin,
    "logo": `${window.location.origin}/logo.png`,
    "description": "Purple Startups helps entrepreneurs turn their ideas into successful businesses with AI-powered tools and resources.",
    "contactPoint": {
      "@type": "ContactPoint",
      "telephone": "+1-555-0123",
      "contactType": "customer service",
      "email": "support@purplestartups.com",
      "availableLanguage": ["English"]
    }
  };

  const handleNonCriticalCSSLoad = (e: React.SyntheticEvent<HTMLLinkElement>) => {
    e.currentTarget.media = 'all';
  };

  return (
    <>
      <Helmet>
        {/* Google Analytics tag - Load after page is interactive */}
        <script type="text/javascript">
          {`
            // Load GA after page is interactive
            window.addEventListener('load', function() {
              // Create and load the GA script
              const gaScript = document.createElement('script');
              gaScript.src = 'https://www.googletagmanager.com/gtag/js?id=G-8NZVJZ1XD4';
              gaScript.async = true;
              document.head.appendChild(gaScript);
              
              // Initialize GA
              window.dataLayer = window.dataLayer || [];
              function gtag(){dataLayer.push(arguments);}
              gtag('js', new Date());
              gtag('config', 'G-8NZVJZ1XD4', {
                'send_page_view': false // We'll send this manually after route changes
              });
            });
          `}
        </script>

        {/* Default meta tags */}
        <title>Purple Startups - AI-Powered Startup Planning Platform</title>
        <meta name="description" content="Purple Startups: AI-powered platform for entrepreneurs. Get expert guidance, business planning tools, and actionable insights to launch and grow your startup. Join 1,000+ founders who turned ideas into thriving businesses." />
        <meta name="keywords" content="startup planning, business strategy, AI startup, entrepreneurship, business planning, market analysis" />

        {/* Preload critical resources */}
        <link
          rel="preload"
          href="/fonts/inter-var.woff2"
          as="font"
          type="font/woff2"
          crossOrigin="anonymous"
          fetchPriority="high"
        />
        
        {/* Preload hero image */}
        <link
          rel="preload"
          href="/images/hero.webp"
          as="image"
          type="image/webp"
          fetchPriority="high"
        />
        
        {/* Preload critical CSS */}
        <link
          rel="preload"
          href="/styles/main.css"
          as="style"
          fetchPriority="high"
        />
        
        {/* Defer non-critical CSS */}
        <link
          rel="stylesheet"
          href="/styles/non-critical.css"
          media="print"
          onLoad={handleNonCriticalCSSLoad}
        />
        
        {/* Add font-display swap for better performance */}
        <style>
          {`
            @font-face {
              font-family: 'Inter var';
              font-weight: 100 900;
              font-display: swap;
              font-style: normal;
              font-named-instance: 'Regular';
              src: url('/fonts/inter-var.woff2') format('woff2');
            }
          `}
        </style>
        
        {/* Structured data */}
        <script type="application/ld+json">
          {JSON.stringify(websiteStructuredData)}
        </script>
        <script type="application/ld+json">
          {JSON.stringify(organizationStructuredData)}
        </script>
      </Helmet>
      <ScrollToTop />
      <Navbar />
      <CookieConsentBanner />
      <Routes>
        {/* Public routes */}
        <Route path="/login" element={<LoginPage />} />
        <Route path="/signup" element={<SignupPage />} />
        <Route path="/forgot-password" element={<ForgotPasswordPage />} />
        <Route path="/reset-password" element={<ResetPasswordPage />} />
        <Route path="/" element={
          <div className="min-h-screen flex flex-col">
            <main className="flex-1 pt-20 px-4 sm:px-6 lg:px-8 max-w-7xl mx-auto">
              {selectedIdea ? (
                <RoadmapView 
                  idea={selectedIdea} 
                  onBack={() => {
                    setSelectedIdea(null);
                    setCurrentPlan(null);
                  }} 
                />
              ) : (
                <>
                  <IdeaGenerator
                    vagueConcept={vagueConcept}
                    setVagueConcept={setVagueConcept}
                    onGenerate={handleGenerateIdeas}
                    isLoading={isLoading}
                  />
                  {ideas.length > 0 && (
                    <IdeaList
                      ideas={ideas}
                      onSelectIdea={setSelectedIdea}
                    />
                  )}
                </>
              )}
            </main>
          </div>
        } />
        
        {/* Add the missing Help route */}
        <Route path="/help" element={<HelpPage />} />
        
        {/* Add the missing FAQ route */}
        <Route path="/faq" element={<FAQPage />} />
        
        {/* Add the missing Contact route */}
        <Route path="/contact" element={<ContactPage />} />
        
        {/* Protected routes */}
        <Route path="/roadmap" element={
          <ProtectedRoute>
            <RoadmapPage />
          </ProtectedRoute>
        } />
        <Route path="/product-roadmap" element={
          <ProtectedRoute>
            <ProductRoadmapPage />
          </ProtectedRoute>
        } />
        
        {/* Admin routes with proper protection */}
        <Route path="/admin" element={
          <AdminRoute>
            <AdminLayout />
          </AdminRoute>
        }>
          <Route index element={<Navigate to="/admin/dashboard" replace />} />
          <Route path="dashboard" element={<AdminDashboardPage />} />
          <Route path="users" element={<UserManagementPage />} />
          <Route path="content" element={<ContentManagementPage />} />
          <Route path="blog" element={<BlogManagementPage />} />
          <Route path="settings" element={<SettingsPage />} />
          <Route path="analytics" element={<AnalyticsPage />} />
          <Route path="contact-messages" element={<ContactMessagesPage />} />
          <Route path="keywords" element={<TrendingKeywordsManager />} />
          <Route path="*" element={<Navigate to="/admin/dashboard" replace />} />
        </Route>
        
        {/* Public content pages */}
        <Route path="/features" element={<FeaturesPage />} />
        <Route path="/pricing" element={<PricingPage />} />
        <Route path="/about" element={<AboutPage />} />
        <Route path="/privacy" element={<PrivacyPage />} />
        <Route path="/terms" element={<TermsPage />} />
        <Route path="/cookies" element={<CookiesPage />} />
        <Route path="/refund" element={<RefundPolicy />} />
        <Route path="/careers" element={<CareersPage />} />
        <Route path="/blog" element={<BlogPage />} />
        <Route path="/ideas" element={<IdeasPage />} />
        
        {/* Dashboard with nested routes */}
        <Route path="/dashboard" element={
          <ProtectedRoute>
            <div>
              <DashboardPage />
            </div>
          </ProtectedRoute>
        }>
          <Route path="profile" element={<ProfilePage />} />
          <Route path="security" element={<SecurityPage />} />
          <Route path="apps" element={<AppsPage />} />
          <Route path="saved-ideas" element={<SavedIdeasPage />} />
          <Route path="notifications" element={<NotificationsPage />} />
          <Route path="billing" element={<BillingPage />} />
          <Route path="api" element={<ApiSettingsPage />} />
          <Route path="help" element={<HelpPage />} />
          <Route index element={<DashboardWelcome />} />
        </Route>

        {/* Apps routes */}
        <Route path="/apps/ai-financial-advisor" element={
          <ProtectedRoute>
            <FinancialProjectionsPage />
          </ProtectedRoute>
        } />

        <Route 
          path="/apps/ai-market-intelligence" 
          element={
            <ProtectedRoute>
              <AIMarketIntelligencePage />
            </ProtectedRoute>
          } 
        />
        <Route 
          path="/apps/ai-marketing-assistant" 
          element={
            <ProtectedRoute>
              <AIMarketingAssistantPage />
            </ProtectedRoute>
          } 
        />
        <Route 
          path="/apps/ai-sales-optimizer" 
          element={
            <ProtectedRoute>
              <AISalesOptimizerPage />
            </ProtectedRoute>
          } 
        />
        <Route 
          path="/apps/ai-product-strategist" 
          element={
            <ProtectedRoute>
              <AIProductStrategistPage />
            </ProtectedRoute>
          } 
        />
        <Route 
          path="/apps/ai-operations-manager" 
          element={
            <ProtectedRoute>
              <AIOperationsManagerPage />
            </ProtectedRoute>
          } 
        />
        <Route 
          path="/apps/ai-legal-assistant" 
          element={
            <ProtectedRoute>
              <AILegalAssistantPage />
            </ProtectedRoute>
          } 
        />

        <Route path="/apps/business-analyst" element={
          <ProtectedRoute>
            <BusinessAnalystPage />
          </ProtectedRoute>
        } />

        <Route path="/apps/business-analyst/:id" element={
          <ProtectedRoute>
            <BusinessAnalystDetailPage />
          </ProtectedRoute>
        } />

        {/* Add Marketing routes */}
        <Route path="/apps/marketing/*" element={
          <ProtectedRoute>
            <MarketingRoutes />
          </ProtectedRoute>
        } />

        {/* Add this new route to handle roadmap with ID parameter */}
        <Route path="/roadmap/:id" element={<RoadmapPage />} />

        {/* Add this route */}
        <Route path="/admin-debug" element={
          <ProtectedRoute>
            <AdminDebugPage />
          </ProtectedRoute>
        } />

        {/* Add this new route */}
        <Route path="/plan/:planId" element={
          <main className="flex-grow">
            <Navbar onSelectPlan={handlePlanSelect} />
            <div className="pt-16">
              <RoadmapView plan={currentPlan} onCloseSidebar={() => setShowSidebar(false)} />
            </div>
          </main>
        } />

        {/* Add this new route for blog post detail */}
        <Route path="/blog/:id" element={<BlogPostDetailPage />} />

        {/* Add this new route for TestimonialPage */}
        <Route path="/testimonials" element={<TestimonialPage />} />

        {/* Add sitemap route */}
        <Route path="/sitemap.xml" element={<SitemapPage />} />

        {/* Add docs route */}
        <Route path="/docs" element={<DocsPage />} />

        {/* Add forum route */}
        <Route path="/forum" element={<CommunityForumPage />} />

        {/* Add knowledge base route */}
        <Route path="/kb" element={<KnowledgeBasePage />} />

        {/* Catch-all route for 404 */}
        <Route path="*" element={<NotFoundPage />} />
      </Routes>
              {/* Only show Footer if not in admin panel or AI apps */}
        {!(window.location.pathname.startsWith('/admin') || 
           window.location.pathname === '/apps/ai-financial-advisor' ||
           window.location.pathname === '/apps/ai-market-intelligence' ||
           window.location.pathname === '/apps/ai-marketing-assistant' ||
           window.location.pathname === '/apps/ai-sales-optimizer' ||
           window.location.pathname === '/apps/ai-product-strategist' ||
           window.location.pathname === '/apps/ai-operations-manager' ||
           window.location.pathname === '/apps/ai-legal-assistant') && <Footer />}
      <UpgradeModal
        isOpen={upgradeModalOpen}
        onClose={() => setUpgradeModalOpen(false)}
        currentPlan={subscription?.plan || 'free'}
      />
    </>
  );
}

// The main App component that provides context providers
function App() {
  return (
    <LocalizationProvider dateAdapter={AdapterDateFns}>
      <HelmetProvider>
        <BrowserRouter>
          <AuthProvider>
            <SubscriptionProvider>
              <SavedPlansProvider>
                <AppContent />
              </SavedPlansProvider>
            </SubscriptionProvider>
          </AuthProvider>
        </BrowserRouter>
      </HelmetProvider>
    </LocalizationProvider>
  );
}

export default App;